/*******************************************
 * I2bshort
 * converts a single GE I-file to bshort (including
 * the header file) given the image dimensions.
 * 
 * USAGE: I2bshort infile rows cols slices outbase
 * 
 * outbase is everything except the ".bshort".
 * Author:  Douglas N. Greve, MGH-NMR
 * April 3, 1999
 *******************************************/
#include<stdio.h>
#include<stdlib.h>
#include<strings.h>

long int FileSizeBytes(FILE *pF);

char infile[128], outbase[128];
char bshortfile[128], hdrfile[128];
FILE *infp,*outfp;
char *c;
int i,j,k;
int offset;
int rows,cols,slices;
 
int main(int argc, char **argv)
{
  if(argc != 6){
    fprintf(stderr,"USAGE: I2bshort infile rows cols slices outbase \n");
    fprintf(stderr,"       outbase is everything except the .bshort\n");
    exit(1);
  }
  sscanf(argv[1],"%s",infile);
  sscanf(argv[2],"%d",&rows);
  sscanf(argv[3],"%d",&cols);
  sscanf(argv[4],"%d",&slices);
  sscanf(argv[5],"%s",outbase);
  
  infp = fopen(infile,"rb");
  if(infp==NULL){
    fprintf(stderr,"Error: cannot open %s\n",infile);
    exit(1);
  } 

  k = FileSizeBytes(infp);
  c = (char *) malloc(k*sizeof(char));
  i = fread(c, sizeof(char), k, infp);
  fclose(infp);

  offset = k - rows*cols*slices*2;

  if(offset < 0){
    fprintf(stderr,"Error: offset (%d) exceeds file size (%d)\n",
	    offset,k);
    exit(1);
  } 
  
  strcpy(bshortfile,outbase);
  strcat(bshortfile,".bshort");
  outfp = fopen(bshortfile,"wb");
  if(outfp==NULL){
    fprintf(stderr,"Error: cannot open outfile %s\n",argv[3]);
    exit(1);
  } 

  i = fwrite((void *)(&c[offset]),sizeof(char),k-offset,outfp);
  fclose(outfp);

  strcpy(hdrfile,outbase);
  strcat(hdrfile,".hdr");
  outfp = fopen(hdrfile,"w");
  fprintf(outfp,"%d %d %d 0\n",rows,cols,slices);
  fclose(outfp);  

  return(0);
}

long int FileSizeBytes(FILE *pF)
{
    long int CurPos, EndPos;
    
    CurPos = ftell(pF);
    fseek(pF,0L,SEEK_END);
    EndPos = ftell(pF);
    fseek(pF,0L,CurPos);
    return(EndPos);
}
