/****************************************************
  Name: bf2bs.c
  USAGE: bf2bs bs_stem <bf_stem>
  Purpose: converts bshort to bfloat. 
  Notes: the input argument is the stem of the bshort 
         file and (optionally) the stem of the bfloat 
         file.  The bfloat output is written to 
         bf_stem.bfloat. If bf_stem is not specified,
         the bfloat is written to bs_stem.bfloat.

  Author: Douglas Greve, MGH-NMR
  Date:   December 9, 1998
*****************************************************/

#include <stdio.h>
#include <string.h>

long int FileSizeBytes(FILE *pF);

FILE *pBSF, *pBFF;
long int nBytes, nVoxels, n;
char bsname[128], bfname[128];
short int *sVoxels, *psV;
float *fVoxels, *pfV;
long int nRead;
long int nWrote;

int main(int argc, char **argv)
{
    if( argc < 2)
    {
	fprintf(stderr,"USAGE: bf2bs bf_stem <bs_stem>\n");
	fprintf(stderr,"  Converts bfloat to bshort\n");
	fprintf(stderr,"  Written by Douglas Greve, 12/9/1998\n");
	exit(-1);
    }

    /* Create the file names */    
    sprintf(bfname,"%s.bfloat",argv[1]);    
    if(argc == 3)
	sprintf(bsname,"%s.bshort",argv[2]);    
    else
	sprintf(bsname,"%s.bshort",argv[1]);

    /**** Open the bfloat file *******/
    pBFF = fopen(bfname,"rb");
    if(pBFF == NULL)
    {
	fprintf(stderr,"Cannot open %s for reading\n",bfname);
	exit(-1);
    }
    /**** Open the bshort file *******/
    pBSF = fopen(bsname,"wb");
    if(pBSF == NULL)
    {
	fprintf(stderr,"Cannot open %s for writing.\n",bsname);
	fclose(pBFF);
	exit(-1);
    }
    
    /* Get the number of bytes in the bshort file */
    nBytes  = FileSizeBytes(pBFF);

    /* Compute the number of voxels */
    nVoxels = nBytes/sizeof(float);

    /* Allocate memory for the bshort voxels */
    sVoxels = (short int *) malloc(sizeof(short int)*nVoxels);
    if(sVoxels == NULL)
    {
	fprintf(stderr,"Cannot allocate %ld for sVoxels\n",nVoxels);
	fclose(pBSF);
	fclose(pBFF);
	exit(-1);	
    }
    /* Allocate memory for the bfloat voxels */
    fVoxels = (float *) malloc(sizeof(float)*nVoxels);
    if(fVoxels == NULL)
    {
	fprintf(stderr,"Cannot allocate %ld for fVoxels\n",nVoxels);
	free(sVoxels);
	fclose(pBSF);
	fclose(pBFF);
	exit(-1);	
    }
    
    /* Read the bfloat voxels in to memeory */
    nRead = fread((void *)fVoxels, sizeof(float), nVoxels, pBFF);
    if(nRead != nVoxels)
    {
	fprintf(stderr,"bfloat: Read Error %ld/%ld Voxels \n",nRead,nVoxels);
	free(sVoxels);
	free(fVoxels);
	fclose(pBSF);
	fclose(pBFF);
	exit(-1);	
    }

    /* Convert bshorts to bfloats */
    psV = sVoxels;
    pfV = fVoxels;
    for(n=nVoxels-1L; n != 0; n--) *(psV++) = (short int) (*(pfV++));

    /* Write the bfloats back to disk */    
    nWrote = fwrite((void *)sVoxels, sizeof(short int), nVoxels, pBSF);

    free(sVoxels);
    free(fVoxels);
    fclose(pBSF);
    fclose(pBFF);

    if(nWrote != nVoxels)
    {
	fprintf(stderr,"bshort: Write Error %ld/%ld Voxels \n",nRead,nVoxels);
	exit(-1);	
    }

    return(0);
}


long int FileSizeBytes(FILE *pF)
{
    long int CurPos, EndPos;
    
    CurPos = ftell(pF);
    fseek(pF,0L,SEEK_END);
    EndPos = ftell(pF);
    fseek(pF,0L,CurPos);
    return(EndPos);
}

