/**************************************************************
 Name:    brescale.c
 Purpose: rescales a bfile
 $Id: brescale.c,v 1.2 1999/06/16 23:01:31 greve Exp $
 **************************************************************/

#include <string.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <sys/stat.h>

long int getfilesize(char *fname);

FILE *infp,*outfp;
long i,j,k,nbytes,n0,n1,m0,m1,Nv;
float *dbf;
float MultFactor;
char  instem[1024];
char  outstem[1024];
char  inbfile[1024],  inhdrfile[1024];
char  outbfile[1024], outhdrfile[1024];
int   fs,ls,ns,slice;
int  rows,cols,frames,endian;

int main(int argc, char **argv)
{
  if(argc != 6){
    fprintf(stderr,"USAGE: brescale multfactor instem outstem firstslice nslices\n");
    fprintf(stderr,"       Version: $Id: brescale.c,v 1.2 1999/06/16 23:01:31 greve Exp $\n");
    exit(1);
  }

  sscanf(argv[1],"%f",&MultFactor);
  sscanf(argv[2],"%s",instem);
  sscanf(argv[3],"%s",outstem);
  sscanf(argv[4],"%d",&fs);
  sscanf(argv[5],"%d",&ns);  

  printf("\n\n------->>>>>>> brescale <<<<<<<<--------------\n");
  printf("$Id: brescale.c,v 1.2 1999/06/16 23:01:31 greve Exp $\n");
  printf("factor  = %g\n",MultFactor);
  printf("instem  = %s\n",instem);
  printf("outstem = %s\n",outstem);  
  printf("firstslice = %3d\n",fs);
  printf("nslices    = %3d\n",ns);

  ls = fs + ns;
  for(slice = fs; slice < ls; slice ++){

    printf("--------- Slice %2d ---------------\n",slice);

    /******** open the input file ******/  
    sprintf(inbfile,"%s_%03d.bfloat",instem,slice);
    printf("  Loading data\n");
    infp = fopen(inbfile,"rb");
    if(infp==NULL){
      fprintf(stderr,"Error: cannot open %s\n",inbfile);
      perror(argv[0]);
      exit(1);
    } 
    /******* get number of bytes in file ********/
    nbytes = getfilesize(inbfile);

    /******* load data **************/
    Nv = nbytes/sizeof(float);
    dbf = (float *) malloc(Nv*sizeof(float));
    i = fread(dbf, sizeof(float), Nv, infp);
    fclose(infp);

    /****** rescale ************/
    printf("  Rescaling data\n");
    for(k=0;k<Nv;k++) dbf[k] *= MultFactor;

    /******** open the output file ******/  
    sprintf(outbfile,"%s_%03d.bfloat",outstem,slice);
    printf("  Saving data\n");
    outfp = fopen(outbfile,"wb");
    if(outfp==NULL){
      fprintf(stderr,"Error: cannot open %s\n",outbfile);
      perror(argv[0]);
      exit(1);
    } 
    i = fwrite(dbf, sizeof(float), Nv, outfp);
    if( i != Nv ){
      fprintf(stderr,"Error: writing %s\n",outbfile);
      perror(argv[0]);    
      exit(1);
    }
    
    /******** open the input hdr file ******/  
    sprintf(inhdrfile,"%s_%03d.hdr",instem,slice);
    printf("  Loading header file\n");
    infp = fopen(inhdrfile,"r");
    if(infp==NULL){
      fprintf(stderr,"Error: cannot open %s\n",inhdrfile);
      perror(argv[0]);    
      exit(1);
    } 
    fscanf(infp,"%d %d %d %d",&rows,&cols,&frames,&endian);
    fclose(infp);
    
    /******** open the output hdr file ******/  
    sprintf(outhdrfile,"%s_%03d.hdr",outstem,slice);
    printf("  Saving header file\n");
    outfp = fopen(outhdrfile,"w");
    if(outfp==NULL){
      fprintf(stderr,"Error: cannot open %s\n",outhdrfile);
      perror(argv[0]);    
      exit(1);
    } 
    fprintf(outfp,"%d %d %d %d\n",rows,cols,frames,endian);
    fclose(outfp);
  }

  printf("------->>>>>>> done <<<<<<<<--------------\n\n");
  return(0);
}

long int getfilesize(char *fname)
{
  struct stat filestat;
  int e;
  
  e = stat(fname, &filestat);
  if(e != 0){
    fprintf(stderr,"ERROR: Input File %s\n",fname);
    fprintf(stderr,"%s\n",strerror(errno));
    exit(1);
  }

  return((long int)filestat.st_size);
}

