/*****************************************************************
Name: brueker2bshort
Purpose: converts from the Brueker MRI scanner output to bshort.
Author: Douglas N. Greve, MGH-MR
Date:   April 12, 1999
$Id: brueker2bshort.c,v 1.1 1999/04/12 22:33:37 greve Exp $
****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>

int nRows, nCols, nFuncSlices, nAnatSlices,nV;
long nTot, s, nRead, InFileSize;
char InFile[1024];
char OutStem[1024];
char OutBFile[1024];
char OutHDRFile[1024];
char *OutDir;


long int getfilesize(char *fname);
char * getoutdir(char *outstem);


main(int argc, char **argv)
{

  int i, j, k, e;
  short int *buf;
  FILE *fin, *fout;

  if(argc != 7){
    fprintf(stderr,"USAGE: brueker2bshort InFile nRows nCols nFuncSlices nAnatSlices OutStem\n");
    fprintf(stderr,"Version: $Id: brueker2bshort.c,v 1.1 1999/04/12 22:33:37 greve Exp $\n");
    exit(1);
  }
  sscanf(argv[1],"%s",InFile);
  sscanf(argv[2],"%d",&nRows);
  sscanf(argv[3],"%d",&nCols);
  sscanf(argv[4],"%d",&nFuncSlices);
  sscanf(argv[5],"%d",&nAnatSlices);
  sscanf(argv[6],"%s",OutStem);

  if(nRows <= 0){
    fprintf(stderr,"nRows = %d, must be > 0\n",nRows);
    exit(1);
  }
  if(nCols <= 0){
    fprintf(stderr,"nCols = %d, must be > 0\n",nCols);
    exit(1);
  }
  if(nFuncSlices <= 0){
    fprintf(stderr,"Number of Functional Slices = %d, must be > 0\n",
	    nFuncSlices);
    exit(1);
  }
  if(nAnatSlices <= 0){
    fprintf(stderr,"Number of Anatomical Slices = %d, must be > 0\n",
	    nAnatSlices);
    exit(1);
  }

  nTot = nRows*nCols*nFuncSlices*nAnatSlices;
  InFileSize = getfilesize(InFile);
  if( InFileSize != nTot*2){
    fprintf(stderr,"File %s has size %ld bytes, expecting %ld\n",
	    InFile,InFileSize,nTot*2);
    exit(1);
  }
  nV = nRows*nCols;
  
  buf = (short int *)malloc( nTot * sizeof(short int));
  fprintf(stderr, "Allocating %ld bytes \n",nTot*2);
  if(buf == NULL){
    fprintf(stderr, "can't allocate %ld memory\n",nTot);
    exit(0);
  }

  fprintf(stderr, "Opening %s \n",InFile);
  if((fin = fopen(InFile, "rb")) == NULL){
    free(buf);
    fprintf(stderr,"can't open file %s\n", InFile);
    fprintf(stderr,"%s\n",strerror(errno));
    exit(0);
  }

  fprintf(stderr, "Reading %s \n",InFile);
  nRead = fread(buf, 2, nTot, fin);
  fclose(fin);

  /*fprintf(stderr,"Items read = %ld\n",nRead);*/
  if(nRead != nTot){
    fprintf(stderr,"ERROR: did not read correct number of items\n");
    exit(1);
  }

  OutDir = getoutdir(OutStem);
  printf("outdir = %s\n",OutDir);
  if(OutDir != NULL){
    e = mkdir(OutDir,(unsigned int)(-1));
    if(e != 0){
      if(errno != EEXIST){
	fprintf(stderr,"ERROR: Could not create directory %s\n",OutDir);
	fprintf(stderr,"%s\n",strerror(errno));
	exit(1);
      }
    }
  }

  for(i = 0 ; i < nAnatSlices;i++){
    sprintf(OutBFile, "%s_%03d.bshort", OutStem,i);
    fout = fopen(OutBFile, "wb");
    if(fout == NULL){
      fprintf(stderr,"Could not open %s for writing\n",OutBFile);
      fprintf(stderr,"%s\n",strerror(errno));
      exit(1);
    }
    
    printf("Writing %s...\n", OutBFile);
    for(j = 0 ; j < nFuncSlices; j++){
      s = nV * (j*nAnatSlices + i);
      /*printf("j = %d, s = %ld, %lx\n",j,s,&buf[s]);*/
      fwrite(&buf[s], 2, nV, fout);
    }
    fclose(fout);

    sprintf(OutHDRFile, "%s_%03d.hdr", OutStem,i);
    fout = fopen(OutHDRFile, "w");
    if(fout == NULL){
      fprintf(stderr,"Could not open %s for writing\n",OutHDRFile);
      exit(1);
    }
    fprintf(fout,"%d %d %d 0\n",nRows,nCols,nFuncSlices);
    fclose(fout);    
  }

  free(buf);

} /* end main() */

/* EOF */


long int getfilesize(char *fname)
{
  struct stat filestat;
  int e;
  
  e = stat(fname, &filestat);
  if(e != 0){
    fprintf(stderr,"ERROR: Input File %s\n",fname);
    fprintf(stderr,"%s\n",strerror(errno));
    exit(1);
  }

  return((long int)filestat.st_size);
}

char * getoutdir(char *outstem)
{
  int lenstem,n;
  char * outdir;
  
  lenstem = strlen(outstem);
  
  n = lenstem-1;
  while(n > -1 && ((char)(outstem[n])) != '/') n--;
  
  if(n == -1) return(NULL);
  
  outdir = (char *) malloc(n*sizeof(char));
  strncpy(outdir,outstem,n);
  
  return(outdir);
}
