/***************************************************
 Name: diffw
 Purpose: compute statistics on the difference between
   two w files (w files are created by paint).
   ********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

int   fread1(int *v, FILE *fp) ;
int   fread2(int *v, FILE *fp) ;
int   fread3(int *v, FILE *fp) ;
int   fread4(float *v, FILE *fp) ;
float freadFloat(FILE *fp) ;

char wname1[256], wname2[256];
int  *vnum1, *vnum2;
float *vval1, *vval2;
float *vval1b, *vval2b;
double sum, sum2, avg, std, e, min,max;
int k,kmax;

int main(int argc, char **argv)
{
  
  int i,ilat;
  int num, num1,num2;
  
  float f;
  float lat;
  FILE *fp;

  if(argc != 2 && argc != 3){
    fprintf(stderr,"USAGE: diffw wfile1 <wfile2>\n");
    exit(1);
  }
  
  strcpy(wname1,argv[1]);

  /** read in the first file ****/
  fp = fopen(wname1,"r");
  if (fp==NULL) {
    fprintf(stderr,"File %s not found\n",wname1);
    exit(1);
  }
  fread2(&ilat,fp);
  fread3(&num1,fp);
  vnum1 = (int *)   malloc(num1*sizeof(int));
  vval1 = (float *) malloc(num1*sizeof(float));
  for (i=0;i<num1;i++)
  {
    fread3(&vnum1[i],fp);
    vval1[i] = freadFloat(fp) ;
  }
  fclose(fp);

  if(argc == 3){
    strcpy(wname2,argv[2]);
    /** read in the second file ****/
    fp = fopen(wname2,"r");
    if (fp==NULL) {
      fprintf(stderr,"File %s not found\n",wname2);
      exit(1);
    }
    fread2(&ilat,fp);
    fread3(&num2,fp);
    vnum2 = (int *)   malloc(num2*sizeof(int));
    vval2 = (float *) malloc(num2*sizeof(float));
    for (i=0;i<num2;i++)
      {
	fread3(&vnum2[i],fp);
	vval2[i] = freadFloat(fp) ;
      }
    fclose(fp);

    /***** fill in holes **********/
    if(vnum1[num1-1] > vnum2[num2-1]) kmax = vnum1[num1-1] ;
    else                              kmax = vnum2[num2-1] ;
  }
  else {
    kmax = vnum1[num1-1] ;
    num2 = num1;
  }

  kmax ++;

  vval1b = (float *) malloc(kmax*sizeof(float));
  memset (vval1b, 0, kmax*sizeof(float));
  for (i=0;i<num1;i++) vval1b[vnum1[i]] = vval1[i];
  free(vval1);
    
  vval2b = (float *) malloc(kmax*sizeof(float));
  memset (vval2b, 0, kmax*sizeof(float));
  if(argc == 3)
    for (i=0;i<num2;i++) vval2b[vnum2[i]] = vval2[i];

  sum  = 0;
  sum2 = 0;
  min  =  1000000000;
  max  = -1000000000;
  for(k=0;k<kmax;k++){
    e = vval1b[k] - vval2b[k];
    sum  += e;
    sum2 += e*e;
    if(min > e) min = e;
    if(max < e) max = e;
  }

  avg = sum/kmax;
  std = sqrt((sum2 - kmax*avg*avg)/(kmax-1.0));

  printf("n = %d, avg = %g, std = %g, min = %g, max = %g\n",
	 kmax, avg,std,min,max);

  return(0);
  
}
/*************************************************************/


int
fread1(int *v, FILE *fp)
{
  unsigned char c;
  int  ret ;

  ret = fread(&c,1,1,fp);
  *v = c;
  return(ret) ;
}


int
fread2(int *v, FILE *fp)
{
  short s;
  int   ret ;

  ret = fread(&s,2,1,fp);
#ifdef Linux
  s = swapShort(s) ;
#endif
  *v = s;
  return(ret) ;
}

int
fread3(int *v, FILE *fp)
{
  unsigned int i = 0;
  int  ret ;

  ret = fread(&i,3,1,fp);
#ifdef Linux
  i = (unsigned int)swapInt(i) ;
#endif
  *v = ((i>>8) & 0xffffff);
  return(ret) ;
}

int
fread4(float *v, FILE *fp)
{
  float f;
  int   ret ;

  ret = fread(&f,4,1,fp);
#ifdef Linux
  f = swapFloat(f) ;
#endif
  *v = f;
  return(ret) ;
}


float
freadFloat(FILE *fp)
{
  float f;
  int   ret ;

  ret = fread(&f,4,1,fp);
#ifdef Linux
  f = swapFloat(f) ;
#endif
  return(f) ;
}

#if 0
MRISreadValues(MRI_SURFACE *mris, char *fname)
{
  int i,k,num,ilat;
  float f;
  float lat;
  FILE *fp;

  fp = fopen(fname,"r");
  if (fp==NULL) 
    ErrorReturn(ERROR_NOFILE, (ERROR_NOFILE,
                               "MRISreadValues: File %s not found\n",fname));
  fread2(&ilat,fp);
  lat = ilat/10.0;

  for (k=0;k<mris->nvertices;k++)
    mris->vertices[k].val=0;
  fread3(&num,fp);
  for (i=0;i<num;i++)
  {
    fread3(&k,fp);
    f = freadFloat(fp) ;
    if (k>=mris->nvertices||k<0)
      printf("MRISreadValues: vertex index out of range: %d f=%f\n",k,f);
    else
    {
      mris->vertices[k].val = f;
    }
  }
  fclose(fp);
  return(NO_ERROR) ;
}

#endif
