#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "pkfutil.h"


int main(int argc, char *argv[]) ;
static int  parse_commandline(int argc, char **argv);
static void check_options(void);
static void print_usage(void) ;
static void usage_exit(void);
static void print_help(void) ;
static void print_version(void) ;
static void argnerr(char *option, int n);

static char vcid[] = "$Id: pkfinder.c,v 1.3 2000/01/06 21:30:36 greve Exp greve $";

char *Progname;

CLUSTER *cl, *cl2;
VOLSS ss;
IMGVOL *iv, *ivpeaks;
CLUSTER_LIST *clist, *clist2, *clist3, *clist4, *clist5, *clist6;
float valthresh, pvalthresh, distthresh;
int   sizethresh, pksign, frame, nopeaks, verbose;
char  *instem;
int   ngrowmax = 1;
int   nbsize = 1;
int   roidef = 0;
int   no1988 = 0;

float col_x0, row_y0, slice_z0;
float colres, rowres, sliceres; 

char CoorDef[20] = "RPI";

int main(int argc, char **argv)
{
  char  stemtemplate[2048];
  char  fname[2048];

  Progname = argv[0] ;
  argc --;
  argv++;

  frame      = 0;
  pksign     = 1;
  pvalthresh = .00001;
  valthresh  = -log(pvalthresh);
  distthresh = 12.0;
  sizethresh = -1;
  instem = "";
  col_x0   = 23;
  row_y0   = 26;
  slice_z0 = 24;
  colres   = 3.125;
  rowres   = 3.125;
  sliceres = 3.125;
  nopeaks  = 0;
  verbose  = 0;

  parse_commandline(argc, argv);
  check_options();

  if(pksign < 0) valthresh *= -1;

  sprintf(stemtemplate,"%s_%%03d",instem);

  iv = LoadBVolumeAsImgVol(stemtemplate,NULL,frame);
  iv->colres   = colres;
  iv->rowres   = rowres;
  iv->sliceres = sliceres;
  iv->col_x0   = col_x0;
  iv->row_y0   = row_y0;
  iv->slice_z0 = slice_z0;

  clist = GetSuperVoxels(iv,valthresh,pksign,nbsize);
  fprintf(stderr,"Found %d voxels over threshold %g\n",
	  clist->nclusters,valthresh);
  clist2 = SortClusterList(clist,iv);

  if(verbose){
    fprintf(stderr,"------------- SupraThresh Vox -------------\n");
    DumpClusterList(stdout,clist2,iv,"%8.4f");
  }

  fprintf(stderr,"Pruning by Distance\n");
  clist3 = PruneByDist(clist2,distthresh,iv);
  fprintf(stderr,"Found %d clusters after dist (%g) prune\n",
	  clist3->nclusters,distthresh);
  if(verbose){
    fprintf(stderr,"------------- Clusters after Pruning by Dist -------------\n");
    DumpClusterList(stderr,clist3,iv,"%8.4f");
  }

  fprintf(stderr,"Growing Clusters\n");
  GrowClusterList(clist3,iv,ngrowmax);
  if(verbose){
    fprintf(stderr,"------------------- Grown Clusters ----------------------\n");
    DumpClusterList(stderr,clist3,iv,"%8.4f");
  }

  fprintf(stderr,"Pruning by Size\n");
  clist4 = PruneBySize(clist3,sizethresh);
  fprintf(stderr,"Found %d clusters after size (%d) prune\n",
	  clist4->nclusters,sizethresh);
  if(verbose){
    fprintf(stderr,"-------------- Clusters after Pruning by Size ------------\n");
    DumpClusterList(stderr,clist4,iv,"%8.4f");
  }

  /*printf("------- Final Cluster List -----------\n");
    DumpClusterList(stdout,clist4,iv,"%8.4f");*/
  /*printf("\n");*/

  clist5 = SortClusterList(clist4,iv);

  fprintf(stderr,"--------------------------------\n");
  PrintClusterReport(stdout, clist5, iv, instem, no1988, CoorDef);

  if(!nopeaks){
    ivpeaks = ClusterList2ImgVol(clist5,iv,roidef);
    sprintf(fname,"%s.peaks",instem);
    SaveImgVolAsBStack(fname,ivpeaks);
  }

  FreeImgVol((void**)&iv);

  return(0);
  exit(0);
}


/* --------------------------------------------- */
static int parse_commandline(int argc, char **argv)
{
  int  nargc = argc, nargs = 0 ;
  char **pargv = argv, *option ;
  
  if(nargc < 1) usage_exit();

  while(nargc > 0){
    option = pargv[0];
    if (!strcmp(option, "-help"))
      print_help() ;
    else if (!strcmp(option, "-version"))
      print_version() ;

    /* ---------------- */
    else if (!strcmp(option, "-cluster")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%d",&sizethresh);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-ngrowmax")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%d",&ngrowmax);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-pval")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%f",&pvalthresh);
      valthresh = -log(pvalthresh);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-distance")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%f",&distthresh);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-i")){
      if(nargc < 2) argnerr(option,1);
      instem = pargv[1];
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-frame")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%d",&frame);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-roidef")){
      roidef = 1;
      nargs = 1;
    }

    /* ---------------- */
    else if (!strcmp(option, "-pos")){
      pksign = 1;
      nargs = 1;
    }

    /* ---------------- */
    else if (!strcmp(option, "-neg")){
      pksign = -1;
      nargs = 1;
    }

    /* ---------------- */
    else if (!strcmp(option, "-abs")){
      pksign = 0;
      nargs  = 1;
    }

    /* ---------------- */
    else if (!strcmp(option, "-xcenter")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%f",&col_x0);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-ycenter")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%f",&row_y0);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-no1988")){
      no1988 = 1;
      nargs = 1;
    }

    /* ---------------- */
    else if (!strcmp(option, "-zcenter")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%f",&slice_z0);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-xres")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%f",&colres);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-yres")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%f",&rowres);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-zres")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%f",&sliceres);
      nargs = 2;
    }

    /* ---------------- */
    else if (!strcmp(option, "-coordef")){
      if(nargc < 2) argnerr(option,1);
      sscanf(pargv[1],"%s",CoorDef);
      nargs = 2;
    }

    /* ---------*/
    else if (!strcmp(option, "-raw")){
      valthresh = pvalthresh;
      nargs = 1;
    }

    /* ----natural log -----*/
    else if (!strcmp(option, "-ln")){
      valthresh = -log(fabs(pvalthresh));
      nargs = 1;
    }

    /* ----log 10 -----*/
    else if (!strcmp(option, "-log10")){
      valthresh = -log10(fabs(pvalthresh));
      nargs = 1;
    }

    /* ------------------------------------*/
    else if (!strcmp(option, "-nopeaks")){
      nopeaks = 1;
      nargs = 1;
    }

    /* ------------------------------------*/
    else if ( !strcmp(option, "-nosort") || 
  	      !strcmp(option, "-noweight")){
      fprintf(stderr,"Ignoring option %s\n",option);
      nargs = 1;
    }


    /* --- for compatiblity with talcoor --- */
    else if (!strcmp(option, "-simple")){
      nargs = 1;
    }

    else if (!strcmp(option, "-verbose")){
      verbose = 1;
      nargs = 1;
    }

    /* ----- unkown option -------- */
    else{
      fprintf(stderr,"ERROR: Option %s unknown\n",option);
      exit(-1);
    }
    nargc -= nargs;
    pargv += nargs;
  }
  return(0);
}
/* ------------------------------------------------------ */
static void usage_exit(void)
{
  print_usage() ;
  exit(1) ;
}
/* --------------------------------------------- */
static void print_usage(void)
{
  fprintf(stderr, "USAGE: %s \n",Progname) ;
  fprintf(stderr, "   -i instem (required)\n");
  fprintf(stderr, "   -frame number of frame in input to process (0) \n");
  fprintf(stderr, "   -cluster minimum cluster size (required) \n");
  fprintf(stderr, "   -ngrowmax grow cluster over ngrowmax box (%d)\n",ngrowmax);
  fprintf(stderr, "   -pval    maximum pval (%g)\n",pvalthresh);
  fprintf(stderr, "   -distance min dist between peaks (%g) mm \n",distthresh);
  fprintf(stderr, "   -xcenter  column at which x=0 (%g) \n",col_x0);
  fprintf(stderr, "   -ycenter  row    at which y=0 (%g) \n",row_y0);
  fprintf(stderr, "   -no1988   dont subtract 12mm from ycenter\n");
  fprintf(stderr, "   -zcenter  slice  at which z=0 (%g) \n",slice_z0);
  fprintf(stderr, "   -xres  distance between columns (%g) mm\n",colres);
  fprintf(stderr, "   -yres  distance between rows    (%g) mm\n",rowres);
  fprintf(stderr, "   -zres  distance between slices  (%g) mm\n",sliceres);
  fprintf(stderr, "   -pos create coordinate list based on positive changes (default)\n");
  fprintf(stderr, "   -neg create coordinate list based on negative changes\n");
  fprintf(stderr, "   -abs create coordinate list based on absolute changes\n");
  fprintf(stderr, "   -nopeaks do not create an instem.peaks.bfloat file\n");
  fprintf(stderr, "   -raw   threshold = pval\n");
  fprintf(stderr, "   -ln    threshold = -ln(pval) (default)\n");
  fprintf(stderr, "   -log10 threshold = -log10(pval)\n");
  fprintf(stderr, "   -verbose print out A LOT of information \n");
  fprintf(stderr, "NOTE: 12 mm is subtracted from ycenter "
                  "in output lists for 1988 atlas\n");
}
/* --------------------------------------------- */
static void print_help(void)
{
  print_usage() ;
  fprintf(stderr, "\nHelp. \n") ;
  exit(1) ;
}
/* --------------------------------------------- */
static void print_version(void)
{
  fprintf(stderr, "%s\n", vcid) ;
  exit(1) ;
}
/* --------------------------------------------- */
static void argnerr(char *option, int n)
{
  if(n==1)
    fprintf(stderr,"ERROR: %s flag needs %d argument\n",option,n);
  else
    fprintf(stderr,"ERROR: %s flag needs %d arguments\n",option,n);
  exit(-1);
}
/* --------------------------------------------- */
static void check_options(void)
{
  if(instem == NULL){
    fprintf(stderr,"An intput stem must be supplied\n");
    exit(1);
  }
  if(sizethresh < 0){
    fprintf(stderr,"Must specify a minimum cluster size\n");
    exit(1);
  }
  return;
}
