/****************************************************
  Name:    synthbfile.c
  USAGE:   synthbfile outstem format fs ns mean namp
  Purpose: simple tool to generate bfiles (either
           bshort or bfloat).

  Author: Douglas Greve, MGH-NMR
  Date:   May 5, 1999
*****************************************************/

#include <stdio.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#define ISOPTION(c)  ((c) == '-')


#ifndef ENDIAN
#define ENDIAN 0
#endif

static int get_option(int argc, char *argv[]);
static void print_version(void);
static void print_usage(void);
/*static void usage_exit(void);*/
static void print_help(void);
static double Gaussian01PDF(void);

int FirstSlice =  0;
int NSlices    = 16;

long nRows      = 64;
long nCols      = 64;
long nPlanes    =  2;
int  dof        =  0;
int  AmpIncr    =  0;

char BFormat[128] = "bfloat\0";
char OutStem[128] = "synth\0";
float Mean = 5.0;
float NoiseAmp = 0.0;
long Seed = 53;
double dtmp;

int cond,Nc = 0;
int Nh;

long Nv,v;
FILE *fpOut,*fpHDR, *fpDOF, *fpDat;

double tpv[10][50], tpvavg[10];
int    ntpv = -1;
char OutFile[256], HDRFile[256];
char DOFFile[256], DatFile[256];
float *data_bfloat;
short  *data_bshort;
void   *data;
int   p,r,c,Endian;
char *Progname ;
int  OutStemSpec = 0;
int  stripe = 0;
char  StripeAxis[24];

int main(int argc, char **argv)
{
  int n, ac,nargs;
  char **av;
  
  Endian = ENDIAN;
  /*printf("Endian = %d\n",Endian);*/

  Progname = argv[0] ;
  if (argc == 1)    print_help() ;

  ac = argc ;
  av = argv ;
  for ( ; argc > 1 && ISOPTION(*argv[1]) ; argc--, argv++)
  {
    nargs = get_option(argc, argv) ;
    argc -= nargs ;
    argv += nargs ;
  }
  if (argc != 1)    print_help() ;
  if (!OutStemSpec){
    fprintf(stderr,"ERROR: Must specify output stem\n");
    print_help() ;
  }

  if(ntpv > nPlanes){
    nPlanes = ntpv;
    for(c=0;c<Nc;c++){
      tpvavg[c] = 0.0;
      for(n=0;n<ntpv;n++) tpvavg[c] += tpv[c][n];
      tpvavg[c] /= (2.0*ntpv);
    }
  }
  else Nc = 1;

  printf("OutStem = %s, Exention = %s\n",OutStem,BFormat);
  printf("nRows = %ld, nCols = %ld, nPlanes = %ld\n",
	 nRows,nCols,nPlanes);
  printf("FirstSlice = %d, nSlices = %d\n",FirstSlice,NSlices);
  printf("Seed = %ld\n",Seed);

  srand48(Seed);
  Nv = nRows*nCols;
  
  if(!strcmp("bshort",BFormat)){
    printf("Using bshort format %d\n",sizeof(short));
    data_bshort = (short *) malloc(Nv*sizeof(short));
    data = (void *) data_bshort;
  }
  else if(!strcmp("bfloat",BFormat)){
    printf("Using bfloat format %d\n",sizeof(float));
    data_bfloat = (float *) malloc(Nv*sizeof(float));
    data = (void *) data_bfloat;
  }
  else{
    fprintf(stderr,"Format %s unrecognized\n",BFormat);
    exit(1);
  }

  Nh = nPlanes;
  sprintf(DatFile,"%s.dat",OutStem);
  if(dof > 0){
    nPlanes = 2*Nc*Nh;

    fpDat = fopen(DatFile,"w");
    if(fpDat == NULL){
      fprintf(stderr,"Could not open %s\n",DatFile);
      perror(argv[0]);
      exit(1);
    }
    fprintf(fpDat,"TR  2\n");
    fprintf(fpDat,"TW 20\n");
    fprintf(fpDat,"TPS 0\n");
    fprintf(fpDat,"Nc  %d\n",Nc);
    fprintf(fpDat,"Nh  %d\n",Nh);
    fprintf(fpDat,"DOF  %d\n",dof);
    fprintf(fpDat,"nRuns  1\n");
    fprintf(fpDat,"Ntp  128\n");
    fprintf(fpDat,"nRows  %ld\n",nRows);
    fprintf(fpDat,"nCols  %ld\n",nCols);
    fprintf(fpDat,"nSkip 0\n");
    fprintf(fpDat,"DTOrder 1\n");    
    fprintf(fpDat,"Rescale 1\n");    
    fprintf(fpDat,"HanRad  0\n");    
    fprintf(fpDat,"NoiseAC  1\n");    
    fprintf(fpDat,"BASeg    0\n");    
    fprintf(fpDat,"GammaFit 0\n");    
    fprintf(fpDat,"gfDelta  0\n");    
    fprintf(fpDat,"gfTau    0\n");    
    fprintf(fpDat,"NullCondId 0\n");    
    
    fclose(fpDat);
  }


  for(n=FirstSlice; n < FirstSlice+NSlices; n++)
    {
      sprintf(OutFile,"%s_%03d.%s",OutStem,n,BFormat);
      sprintf(HDRFile,"%s_%03d.hdr",OutStem,n);
      sprintf(DOFFile,"%s_%03d.dof",OutStem,n);
      printf("%02d %s %s\n",n,OutFile,HDRFile);

      fpOut = fopen(OutFile,"wb");
      if(fpOut == NULL){
	fprintf(stderr,"Could not open %s\n",OutFile);
	perror(argv[0]);
	exit(1);
      }
 
      for(p=0;p<nPlanes;p++){
	if(ntpv > 0) {
	  if(dof > 0){
	    r = p%(2*Nh);
	    c = (int)p/(2*Nh);
	    if(r < Nh) Mean = tpv[c][r];
	    else       Mean = tpvavg[c];
	  }
	  else  Mean = tpv[0][p];
	}

	if(!strcmp("bshort",BFormat)){
	  for(v=0L;v<Nv;v++) {
	    if(!stripe || n%2 == 1){
              /*dtmp = Mean + 2.0*NoiseAmp*(drand48()-0.5);*/
	      dtmp = Mean + NoiseAmp*Gaussian01PDF();
	      if(AmpIncr) dtmp += n;
	      if(dtmp < 0) dtmp = 0.0;
	      if(dtmp > 32000) dtmp = 32000.0;
	      data_bshort[v] = (short int)(rint(dtmp));
	    }
	    else{
	      if(n%2 == 0) data_bshort[v] = (short int) 0;
	    }
	  }
	  fwrite(data_bshort, sizeof(short int), Nv, fpOut);
	}
	else{ /* bfloat format */
	  for(v=0L;v<Nv;v++) {
	    /*dtmp = Mean + 2.0*NoiseAmp*(drand48()-0.5);*/
	    dtmp = Mean + NoiseAmp*Gaussian01PDF();	    
	    if(AmpIncr) dtmp += n;
	    if(stripe && n%2 == 1) dtmp *= -1.0;
	    data_bfloat[v] = (float)dtmp;
	  }
	  fwrite(data_bfloat, sizeof(float), Nv, fpOut);
	}
      }
      fclose(fpOut);

      fpHDR = fopen(HDRFile,"w");
      if(fpHDR == NULL){
	fprintf(stderr,"Could not open %s\n",HDRFile);
	perror(argv[0]);
	exit(1);
      }
#if SYSTEM == Linux
      fprintf(fpHDR,"%ld %ld %ld %d\n",nRows,nCols,nPlanes,Endian);
#else
      fprintf(fpHDR,"%ld %ld %ld %d\n",nRows,nCols,nPlanes,0);
#endif
      fclose(fpHDR);

      if(dof > 0){
	fpDOF = fopen(DOFFile,"w");
	if(fpDOF == NULL){
	  fprintf(stderr,"Could not open %s\n",DOFFile);
	  perror(argv[0]);
	  exit(1);
	}
        for(cond=0;cond<Nc;cond++)
	  fprintf(fpDOF,"%d %d %d\n",cond,dof,dof-1);
	
	fclose(fpDOF);
      }

    }
  return(0);
}

static int get_option(int argc, char *argv[])
{
  int  nargs = 0,k ;
  char *option, axis ;

  if(argc < 3) print_help();
  
  option = argv[1] + 1 ;            /* past '-' */
  if (!strcmp(option, "-help"))
    print_help() ;
  else if (!strcmp(option, "-version"))
    print_version() ;
  else switch (tolower(*option))
  {
  case 'o':
    strcpy(OutStem,argv[2]) ;
    nargs = 1 ;
    OutStemSpec = 1;
    break ;
  case 'e':
    strcpy(BFormat,argv[2]) ;
    nargs = 1 ;
    break ;
  case 'd':
    sscanf(argv[2],"%d",&dof);    
    nargs = 1 ;
    break ;
  case 'r':
    sscanf(argv[2],"%ld",&nRows);
    nargs = 1 ;
    break ;
  case 'c':
    sscanf(argv[2],"%ld",&nCols);
    nargs = 1 ;
    break ;
  case 'p':
    sscanf(argv[2],"%ld",&nPlanes);
    nargs = 1 ;
    break ;
  case 't':
    k = 0;
    while(! ISOPTION(*argv[k+2]) ){
      sscanf(argv[k+2],"%lf",&tpv[Nc][k]);
      k ++;
      if(k+2 >= argc) break;
    }
    if(k==0){
      fprintf(stderr,"-t option must be followed by at least one value\n");
      exit(1);
    }
    Nc++;
    ntpv = k;
    nargs = k ;
    break ;
  case 'f':
    sscanf(argv[2],"%d",&FirstSlice);
    nargs = 1 ;
    break ;
  case 'n':
    sscanf(argv[2],"%d",&NSlices);
    nargs = 1 ;
    break ;
  case 's':
    sscanf(argv[2],"%ld",&Seed);
    nargs = 1 ;
    break ;
  case 'a':
    sscanf(argv[2],"%f",&NoiseAmp);
    nargs = 1 ;
    break ;
  case 'm':
    sscanf(argv[2],"%f",&Mean);
    nargs = 1 ;
    break ;
  case 'l':
    sscanf(argv[2],"%s",StripeAxis);
    axis = tolower(StripeAxis[0]);
    if(axis != 'x' && axis != 'y' && axis != 'z'){
      printf("Error: axis must be either x, y, or z (%c)",axis);
      exit(1);
    }
    stripe = 1;
    nargs = 1 ;
    break ;
  case 'i':
    sscanf(argv[2],"%d",&AmpIncr);
    nargs = 1 ;
    break ;
  default:
    fprintf(stderr, "unknown option %s\n", argv[1]) ;
    exit(1) ;
    break ;
  }

  return(nargs) ;
}

/*
static void
usage_exit(void)
{
  print_usage() ;
  exit(1) ;
}
*/

static void
print_usage(void)
{
  fprintf(stderr, 
          "usage: %s [options] -o outstem\n",Progname) ;
  fprintf(stderr, "\toptions are:\n") ;
  fprintf(stderr, "\t-e extention (bfloat)\n") ;
  fprintf(stderr, "\t-d dof (0 - do not gen dof file)\n") ;
  fprintf(stderr, "\t-f first anat slice (0)\n") ;
  fprintf(stderr, "\t-n number of anat slices (16)\n") ;
  fprintf(stderr, "\t-r number of rows (64)\n") ;
  fprintf(stderr, "\t-c number of cols (64)\n") ;
  fprintf(stderr, "\t-p number of time points (2)\n") ;
  fprintf(stderr, "\t-t specify values of time points\n") ;
  fprintf(stderr, "\t-m average of all voxels (%g)\n",Mean) ;
  fprintf(stderr, "\t-a noise amp for all voxels (%g)\n",NoiseAmp) ;
  fprintf(stderr, "\t-l axis : alternate average for slices along x,y, or z\n");
  fprintf(stderr, "\t-s random seed (53)\n") ;
  fprintf(stderr, "\t-i inc: increment the amplitude by inc for each slice (0)\n") ;

}

static void
print_help(void)
{
  print_usage() ;
  fprintf(stderr, 
          "\nThis program synthesizes simple bshort or bfloat files\n\n") ;
  exit(1) ;
}

static void print_version(void)
{
  fprintf(stderr, "$Id$ \n") ;
  exit(1) ;
}




/*********************************************************
 * Name:    Gaussian01PDF()
 * Purpose: generates random numbers that obey a gaussian 
 *          distribution with zero mean and std dev of 1:
 *              pdf(x) = e^(x^2/2)/sqrt(2pi)
 ************************************************************/
double Gaussian01PDF(void)
{
  double v1,v2,r2;

  do
    {
      v1 = 2.0 * drand48() - 1.0;
      v2 = 2.0 * drand48() - 1.0;
      r2 = v1*v1 + v2*v2;
    } 
  while( r2 > 1.0);

  return( v1 * sqrt( -2.0 * log(r2)/r2 ));
}

