function R = fast_acorr(x)
% R = fast_acorr(x)
%
% Computes the autocorrelation of x using an FFT.  This should
% produce the same results as the native matlab function xcorr
% when invoked as xcorr(x,nMaxLag,'unbiased'), except that xcorr()
% is slow and will not compute a separate R for each column of x.
%

if(nargin ~= 1)
  msg = 'USAGE: R = fast_acorr(x)';
  qoe(msg);error(msg);
end

% Get dims of x %
[ntrs ncols] = size(x);

% FFT of X %
fftx = fft(x,2*ntrs);

% Correlation in time is element-by-conj(element) mult in frequency %
% Note: Convolution would not use conj().
fftx2 = fftx .* conj(fftx); 

% Inverse FFT to get back to time-domaine
R = real(ifft(fftx2));

% Only keep the first ntrs
R = R(1:ntrs,:);

% Divide by the maximum for each column
Rmax = max(R);
iz = find(Rmax==0);
Rmax(iz) = 10^10;
R = R./repmat(Rmax,[ntrs 1]);

return;
