function [vSig, pSig] = fmri_stxgrinder(Test, hAvg, VoxVar, Ch, DOF, RM, q)
%
% [vSig, <pSig>] = fmri_stxgrinder(Test, hAvg, VoxVar, Ch, DOF, RM, q)
%
%
% $Id: fmri_stxgrinder.m,v 1.5 2001/12/13 22:57:16 greve Exp greve $
%


if(nargin < 6)
 msg = ...
'USAGE: [vSig, pSig] = fmri_stxgrinder(Test, hAvg, VoxVar, Ch,DOF, RM , <q>)';
 qoe(msg);error(msg);
end

%%%% -------  Determine the Test Type ------------ %%%%%%
if(strncmp('T',upper(Test),1))     TestId = 0; % t-Test
elseif(strncmp('F',upper(Test),1)) TestId = 1; % F-Test
else  error(sprintf('Test Type %s is unknown',Test));
end

%%% -- get the dimensions --- %%%
nRows = size(hAvg,1);
nCols = size(hAvg,2);
nT    = size(hAvg,3);
nV = nRows * nCols;

%% ---- Subtract the Ideal, if needed ----- %%%%%
if( nargin == 6 )
  h = hAvg;
else
  if(size(q,1) ~= size(hAvg,3))
    error('hAvg and q must have the same number of rows');
  end
  tmp = reshape(repmat(q,[nV 1]), [nRows nCols nT]);
  h = hAvg - tmp;
  clear tmp;
end

ind0 = find(VoxVar==0);
l0 = length(ind0);
fprintf(1,'  nVoxels with VoxVar=0: %3d\n',l0);
if(l0 == prod(size(VoxVar)))
  fprintf(1,'INFO: All voxels are zero\n');
  if(TestId == 0)
    sz = [nRows nCols size(RM,1)];
  else
    sz = [nRows nCols 1];
  end
  pSig = ones(sz);
  vSig = zeros(sz);
  return;
end
if(l0 ~= 0)
  indnz = find(VoxVar ~= 0);
  VoxVar_min = min(reshape1d(VoxVar(indnz)));
  VoxVar(ind0) = VoxVar_min;
  fprintf(1,'  Setting zeros to min=%g\n',VoxVar_min);
end


%%%% ----- reshape for easier processing ----- %%%%%
h     = reshape(h,[nV nT])'; %'
VoxVarB = reshape(VoxVar,[nV 1])'; %'

%%% ---- Compute inv of DOF/Desgin covariance mtx --- %%%%
RChRt   = RM * Ch * RM'; %'
ind = find(RChRt==0);
RChRt(ind) = .0000000000001;

%%% --- Perform Tests --- %%%
if(TestId == 0) % t-Test
  fprintf('INFO: performing t-test\n');
  dRChRt = diag(RChRt);
  vSig = (RM * h) ./ sqrt(dRChRt * VoxVarB);

  if(nargout == 2)
    pSig = tTest(DOF,reshape1d(vSig),300); % dof>300 --> normal approx %
    pSig = reshape(pSig,size(vSig));
  end

else  % F-Test
  fprintf('INFO: performing F-test\n');
  if(strcmp('FM',upper(Test))) 
    dRChRt = diag(RChRt);
    vSig = (RM * h).^2 ./ (dRChRt * VoxVarB);
    J = 1;
  else
    RvvR = RM' * inv(RChRt) * RM; %'
    [J Nh] =  size(RM); % Rows of RM %
    if(Nh==1) vSig =  (((h' * RvvR)' .*h) ./ VoxVarB)/J;
    else      vSig =  ((sum((h' * RvvR)' .* h)) ./ VoxVarB)/J;
    end
  end

  if(nargout == 2)
    pSig = FTest(J,DOF,reshape1d(vSig),1000); % 1500 = maxdof2
    pSig = reshape(pSig,size(vSig));
  end
end

%% Reshape into image dimensions %%
vSig = reshape(vSig',[nRows nCols size(vSig,1)]); %'
if(nargout == 2)
  pSig = reshape(pSig',[nRows nCols size(pSig,1)]); %'
end

return;
